@@ -42,6 +42,18 @@
 
 WX_DEFINE_OBJARRAY(ArrayOfNode);
 
+bool IsBlockDev( int FD ){
+	struct stat *sbufptr = new struct stat;
+   fstat( FD, sbufptr );
+   if(S_ISBLK( sbufptr->st_mode )
+#ifdef __WXMSW__
+		or (sbufptr->st_mode==0)	//Enable block size detection code on windows targets,
+#endif
+		)
+		return true;
+	return false;
+	}
+
 int FDtoBlockSize( int FD ){
 	int block_size=0;
 #ifdef __WXGTK__
@@ -88,6 +100,11 @@ FAL::FAL(wxFileName& myfilename, FileAccessMode FAM, unsigned ForceBlockRW ){
 	ProcessID=-1;
 	get_ptr = put_ptr = 0;
 	OSDependedOpen( myfilename, FAM, ForceBlockRW  );
+
+#if _FSWATCHER_ //Only GTK port is working good on detect changes of file.
+	if(not myfilename.GetFullPath().Lower().StartsWith( wxT("-pid=")))
+	   wxFileSystemWatcher::Add( myfilename.GetFullPath(), wxFSW_EVENT_MODIFY );
+#endif
 	}
 
 #ifdef __WXMSW__
@@ -112,7 +129,7 @@ bool FAL::OSDependedOpen(wxFileName& myfilename, FileAccessMode FAM, unsigned Fo
 		BlockRWSize=driveInfo.BytesPerSector;
 		BlockRWCount=driveInfo.TracksPerCylinder*driveInfo.SectorsPerTrack*driveInfo.Cylinders.QuadPart;
 
-		int fd = _open_osfhandle((long) hDevice, 0);
+		int fd = _open_osfhandle(reinterpret_cast<intptr_t>(hDevice), 0);
 	#ifdef _DEBUG_
 		std::cout<< "Win Device Info:\n" << "Bytes per sector = " <<  BlockRWSize << "\nTotal number of bytes = " << BlockRWCount << std::endl;
 	#endif
@@ -120,7 +137,7 @@ bool FAL::OSDependedOpen(wxFileName& myfilename, FileAccessMode FAM, unsigned Fo
 		return true;
 		}
 	if( not myfilename.IsFileReadable() ){
-		wxMessageBox(wxString(_("File is not readable by permissions."))+wxT("\n")+_("Please change file permissons or run this program with root privileges"),_("Error"), wxOK|wxICON_ERROR);
+		wxMessageBox(wxString(_("File is not readable by permissions."))+wxT("\n")+_("Please change file permissons or run this program with root privileges."),_("Error"), wxOK|wxICON_ERROR);
 		return false;
 		}
 
@@ -130,7 +147,7 @@ bool FAL::OSDependedOpen(wxFileName& myfilename, FileAccessMode FAM, unsigned Fo
 
 bool FAL::OSDependedOpen(wxFileName& myfilename, FileAccessMode FAM, unsigned ForceBlockRW){
 	struct stat fileStat;
-   bool DoFileExists = (stat(myfilename.GetFullPath().To8BitData(),&fileStat) >= 0);
+   bool DoFileExists = (stat((const char*) myfilename.GetFullPath().fn_str(),&fileStat) >= 0);
 
 	//Handling Memory Process Debugging Here
 	if(myfilename.GetFullPath().Lower().StartsWith( wxT("-pid="))){
@@ -157,7 +174,11 @@ bool FAL::OSDependedOpen(wxFileName& myfilename, FileAccessMode FAM, unsigned Fo
 
 	//Owning file
 	else if( not myfilename.IsFileReadable() and DoFileExists ){ // "and myfilename.FileExist()" not used because it's for just plain files, not for block ones.
-		if( wxCANCEL == wxMessageBox(wxString(_("File is not readable by permissions."))+wxT("\n")+_("Do you want to own the file?"),_("Error"), wxOK|wxCANCEL|wxICON_ERROR) )
+		if( wxCANCEL == wxMessageBox(wxString(_("File is not readable by permissions."))+ wxT("\n")+
+												        _("Please change file permissons or run this program with root privileges.")+wxT("\n")+
+												        _("You can also try to own the file temporarily (requires root's password.)")+wxT("\n")+wxT("\n")+
+												        _("Do you want to own the file?"),_("Error"), wxOK|wxCANCEL|wxICON_ERROR) )
+
 			return false;
 
 		wxArrayString output,errors;
@@ -226,14 +247,17 @@ bool FAL::FALOpen(wxFileName& myfilename, FileAccessMode FAM, unsigned ForceBloc
 		else
 			Open(myfilename.GetFullPath(), wxFile::read_write);
 
-		if( ForceBlockRW )
-			BlockRWCount=wxFile::Length()/ForceBlockRW;
-
 		if(not IsOpened()){
 			file_access_mode = AccessInvalid;
 			wxMessageBox( _("File cannot open."),_("Error"), wxOK|wxICON_ERROR );
 			return false;
 			}
+		if(IsBlockDev( wxFile::fd() )){
+			BlockRWSize=FDtoBlockSize( wxFile::fd() );
+			BlockRWCount=FDtoBlockCount( wxFile::fd() );
+			}
+		else if( ForceBlockRW )
+			BlockRWCount=wxFile::Length()/ForceBlockRW;
 
 		return true;
 		}
@@ -620,7 +644,25 @@ long FAL::ReadR( unsigned char* buffer, unsigned size, uint64_t from, ArrayOfNod
 	///Getting Data from bellow layer.
 	if( PatchIndice == 0 )	//Deepest layer
 		{
-		if ( BlockRWSize > 0 ){
+		//Block Read/Write mechanism.
+		//if( 0 )//for debugging
+
+#ifdef __WXGTK__ //Linux file read speed up hack for Block devices.
+		//Block read code just allowed for memory devices under linux.
+		//Because you can read arbitrary locations from block devices at linux.
+		//Kernel handle the job...
+		//This hack increase reading speed from 164MB to 196MB on my SSD with using read 4MB buffer due use of memcmp.
+		//(Max disk rw is 230MB/s)
+		if( ProcessID >=0 )
+#else
+		if( BlockRWSize > 0 )
+#endif // __WXGTK__
+			{
+			///NOTE:This function just read +1 more sectors and copies to buffer via memcpy, thus inefficient, at least for SSD's.
+			///TODO:Need to read 1 sector at start to bfs, than read to buffer directly and read one more sector to bfr.
+			///Than we could copy readed sectors via memcpy which supposed to increase copy speed some percent.
+
+			//Start & end sector and shift calculation.
 			uint64_t StartSector = from / BlockRWSize;
 			unsigned StartShift = from - (from / BlockRWSize)*BlockRWSize;
 			uint64_t EndSector = (from + size)/BlockRWSize;
@@ -631,6 +673,8 @@ long FAL::ReadR( unsigned char* buffer, unsigned size, uint64_t from, ArrayOfNod
 		   int rd_size = (EndSector - StartSector + 1)*BlockRWSize; //+1 for read least one sector
 			char *bfr = new char[rd_size];
 			int rd=0;
+
+			//If reading from a process memory
 			if ( ProcessID >=0 ){
 				long word=0;
 				//unsigned long *ptr = (unsigned long *) buffer;
@@ -642,10 +686,13 @@ long FAL::ReadR( unsigned char* buffer, unsigned size, uint64_t from, ArrayOfNod
 					}
 				#endif
 				}
-			else{
+			//Reading from a file
+			else
+				{
 				wxFile::Seek(StartSector*BlockRWSize);
 				rd = wxFile::Read( bfr, rd_size);
 				}
+			//Here, we adjust shifting by copying bfr to buffer. Inefficient but easy to programe.
 			memcpy(buffer, bfr+StartShift, wxMin(wxMin( rd, rd_size-StartShift) , size)); //wxMin protects file ends.
 			delete [] bfr;
 			return wxMin(wxMin( rd, rd_size-StartShift), size);
@@ -685,10 +732,10 @@ long FAL::InjectionPatcher(uint64_t current_location, unsigned char* data, int s
 	* first partition                     = xxx *
 	********************************************/
 	if( PatchArray->Item(PatchIndice-1) -> size > 0 and PatchArray->Item(PatchIndice-1)->flag_inject ){
-		DiffNode *Inject_Node = PatchArray->Item(PatchIndice-1);	//For easy reading of code
+		DiffNode *Inject_Node = PatchArray->Item(PatchIndice-1);	//For easy debugging
 		///If injection starts behind, so we needed to read bytes from before to show correct location.
 		///State ...().,,,[...]..... -> ...(...).[,,,].
-		if( Inject_Node->end_offset() < current_location ){
+		if( Inject_Node->end_offset() <= current_location ){
 			return ReadR( data, size, current_location - Inject_Node->size, PatchArray, PatchIndice-1 );
 			}
 		///State ...(),,,[....]..... -> ...(..[xx),,],..
@@ -700,10 +747,10 @@ long FAL::InjectionPatcher(uint64_t current_location, unsigned char* data, int s
 				int first_part = Inject_Node->end_offset() - current_location; // First part size
 				memcpy( data, Inject_Node->new_data + movement, first_part );  // Copy first part to buffer
 				int read_size = ReadR( data+first_part, size-first_part, Inject_Node->start_offset, PatchArray, PatchIndice-1 ); //Than copy second part.
-				return read_size+first_part;
+				return wxMin(read_size+first_part, size);
 			}
 
-		///State ...(),,,[...].).... -> ...(...[...].),,,
+		///State ...(),,,[...]..... -> ...(...[...].),,,
 		else if( Inject_Node->start_offset <= current_location
 					and Inject_Node->end_offset() >= current_location + size
 					){
@@ -720,7 +767,7 @@ long FAL::InjectionPatcher(uint64_t current_location, unsigned char* data, int s
 				int read_size = ReadR( data, first_part, current_location, PatchArray, PatchIndice-1 ); //Read First Part from lower layer
 				memcpy( data+first_part, Inject_Node->new_data, Inject_Node->size );  // Copy second part to buffer
 				read_size += ReadR( data+first_part+Inject_Node->size, size-(first_part+Inject_Node->size), current_location+first_part, PatchArray, PatchIndice-1 ); //Read latest chunk
-				return read_size+Inject_Node->size;
+				return wxMin( read_size+Inject_Node->size , size );
 			}
 		///State ...[...(),,,]... -> ...[xx(...]...),,,...
 		else if( Inject_Node->start_offset > current_location
@@ -730,7 +777,7 @@ long FAL::InjectionPatcher(uint64_t current_location, unsigned char* data, int s
 				int first_part = Inject_Node->start_offset - current_location; // First part size
 				int read_size = ReadR( data, first_part, current_location, PatchArray, PatchIndice-1 ); //Read First Part from lower layer
 				memcpy( data+first_part, Inject_Node->new_data, size-first_part );  // Copy second part to buffer
-				return read_size+Inject_Node->size;
+				return wxMin(read_size+Inject_Node->size, size);
 			}
 		///State ...[...]...(...)... -> ...[...]...()...
 		else{
@@ -750,7 +797,7 @@ long FAL::DeletionPatcher(uint64_t current_location, unsigned char* data, int si
 	* first partition                    = xxx *
 	*******************************************/
 	if( PatchArray->Item(PatchIndice-1) -> size < 0 ){
-		DiffNode *Delete_Node = PatchArray->Item(PatchIndice-1);	//For easy reading of code
+		DiffNode *Delete_Node = PatchArray->Item(PatchIndice-1);	//For easy debugging
 		///If deletion starts behind, so we needed to read bytes from after to show correct location.
 		///State ...(...)...[...],,,. -> ...()......[,,,].
 		///State ...(..[..).].,,,.... -> ...()..[,,,]....
@@ -759,7 +806,7 @@ long FAL::DeletionPatcher(uint64_t current_location, unsigned char* data, int si
 			return ReadR( data, size, current_location - Delete_Node->size, PatchArray, PatchIndice-1 );
 			}
 		///State ...[xxx(...)...],,,... -> ...[xxx()...,,,]...
-		///State ...[xxx(...]...),,,... -> ...[xxx()...,,,]...
+		///State ...[xxx(...]...),,,... -> ...[xxx(),,,...]...
 		else if( Delete_Node->start_offset > current_location and Delete_Node->start_offset < current_location + size ){
 			int first_part_size = Delete_Node->start_offset - current_location;
 			int read_size=0;
