@@ -32,6 +32,9 @@
 //    EVT_KILL_FOCUS( wxHexCtrl::OnKillFocus )
 //END_EVENT_TABLE()
 
+#include <wx/arrimpl.cpp> // this is a magic incantation which must be done!
+WX_DEFINE_OBJARRAY(wxArrayUINT64);
+
 HexEditorCtrl::HexEditorCtrl(wxWindow* parent, int id, const wxPoint& pos, const wxSize& size, long style):
 	HexEditorCtrlGui(parent, id, pos, size, wxTAB_TRAVERSAL){
 	select = new class Select( GetEventHandler() );
@@ -60,7 +63,15 @@ HexEditorCtrl::HexEditorCtrl(wxWindow* parent, int id, const wxPoint& pos, const
 HexEditorCtrl::~HexEditorCtrl( void ){
 	Dynamic_Disconnector();
 	Clear();
-	MainTagArray.Clear();
+
+	WX_CLEAR_ARRAY(MainTagArray)
+	WX_CLEAR_ARRAY(HighlightArray)
+   WX_CLEAR_ARRAY(CompareArray)
+
+   MainTagArray.Shrink();
+   HighlightArray.Shrink();
+   CompareArray.Shrink();
+
 	delete select;
 	delete offset_scroll;
 	}
@@ -73,6 +84,9 @@ void HexEditorCtrl::Dynamic_Connector(){
 	this->Connect( wxEVT_KILL_FOCUS, wxFocusEventHandler(HexEditorCtrl::OnKillFocus),NULL, this);
 	hex_ctrl->Connect( wxEVT_KILL_FOCUS, wxFocusEventHandler(HexEditorCtrl::OnKillFocus),NULL, this);
 	text_ctrl->Connect( wxEVT_KILL_FOCUS, wxFocusEventHandler(HexEditorCtrl::OnKillFocus),NULL, this);
+	hex_ctrl->Connect( wxEVT_SET_FOCUS, wxFocusEventHandler(HexEditorCtrl::OnFocus),NULL, this);
+	text_ctrl->Connect( wxEVT_SET_FOCUS, wxFocusEventHandler(HexEditorCtrl::OnFocus),NULL, this);
+
 
 	offset_ctrl->Connect( wxEVT_RIGHT_DOWN,	wxMouseEventHandler(HexEditorCtrl::OnMouseRight),NULL, this);
 	offset_ctrl->Connect( wxEVT_LEFT_DOWN,	wxMouseEventHandler(HexEditorCtrl::OnMouseLeft),NULL, this);
@@ -109,7 +123,6 @@ void HexEditorCtrl::Dynamic_Disconnector(){
 	}
 
 //-----READ/WRITE FUNCTIONS-------//
-
 void HexEditorCtrl::ReadFromBuffer( uint64_t position, unsigned lenght, char *buffer, bool cursor_reset, bool paint ){
 	if( lenght==4294967295LL ){
 		std::cout << "Buffer has no data!" << std::endl;
@@ -135,18 +148,18 @@ void HexEditorCtrl::ReadFromBuffer( uint64_t position, unsigned lenght, char *bu
 	if(sector_size > 1){
 		offset_ctrl->sector_size=sector_size;
 		unsigned draw_line=sector_size-(page_offset%sector_size);
-		hex_ctrl->ThinSeperationLines.Clear();
-		text_ctrl->ThinSeperationLines.Clear();
+		hex_ctrl->ThinSeparationLines.Clear();
+		text_ctrl->ThinSeparationLines.Clear();
 			do{
-			hex_ctrl->ThinSeperationLines.Add( 2*draw_line );
-			text_ctrl->ThinSeperationLines.Add( draw_line );
+			hex_ctrl->ThinSeparationLines.Add( 2*draw_line );
+			text_ctrl->ThinSeparationLines.Add( draw_line );
 			draw_line += sector_size;
 			}while (draw_line < lenght );
 		}
 
-	if(ProcessRAMMap.Count())
-		hex_ctrl->ThinSeperationLines.Clear();
-		text_ctrl->ThinSeperationLines.Clear();
+	if(ProcessRAMMap.Count()){
+		hex_ctrl->ThinSeparationLines.Clear();
+		text_ctrl->ThinSeparationLines.Clear();
 		//Notice that, ProcessRAMMap is SORTED.
 		for( unsigned i=0; i < ProcessRAMMap.Count(); i++ ){
 			uint64_t M = ProcessRAMMap.Item(i);
@@ -157,10 +170,11 @@ void HexEditorCtrl::ReadFromBuffer( uint64_t position, unsigned lenght, char *bu
 				and (M <= page_offset + ByteCapacity()) ){
 
 				int draw_line = M - page_offset;
-				hex_ctrl->ThinSeperationLines.Add( 2*draw_line );
-				text_ctrl->ThinSeperationLines.Add( draw_line );
+				hex_ctrl->ThinSeparationLines.Add( 2*draw_line );
+				text_ctrl->ThinSeparationLines.Add( draw_line );
 				}
 			}
+		}
 
 	hex_ctrl->SetBinValue(buffer, lenght, false );
 	//text_ctrl->ChangeValue(text_string, false);
@@ -194,8 +208,9 @@ void HexEditorCtrl::HexCharReplace(long hex_location, const wxChar chr){
 //-----MENUS--------//
 
 void HexEditorCtrl::ShowContextMenu( const wxMouseEvent& event ){
+	//Non operational code... HexEditor::ShowContextMenu() used instead
 	wxMenu menu;
-	unsigned TagPosition=0;
+	uint64_t TagPosition=0;
 	if( event.GetEventObject() == hex_ctrl )
 		TagPosition = page_offset + (hex_ctrl->PixelCoordToInternalPosition( event.GetPosition() ) / 2);
 	if( event.GetEventObject() == text_ctrl )
@@ -218,14 +233,14 @@ void HexEditorCtrl::ShowContextMenu( const wxMouseEvent& event ){
 	wxPoint pos = event.GetPosition();
 	wxWindow *scr = static_cast<wxWindow*>( event.GetEventObject() );
 	pos += scr->GetPosition();
-    PopupMenu(&menu, pos);
+   PopupMenu(&menu, pos);
 	}
 
 //-----VISUAL FUNCTIONS------//
 void HexEditorCtrl::SetFont( wxFont f ){
 	stdfont = f;
 	m_static_offset->SetFont( stdfont );
-	m_static_adress->SetFont( stdfont );
+	m_static_address->SetFont( stdfont );
 	m_static_byteview->SetFont( stdfont );
 	SetStyle();
 	}
@@ -294,8 +309,10 @@ bool HexEditorCtrl::Selector(){
 	if( not (FocusAt == hex_ctrl || FocusAt == text_ctrl) ){		//Checks If selecton from hex or text control
 		#ifdef _DEBUG_
 		std::cout << "Selector without focus captured." << std::endl;
+		std::cout << "use last focus as a work-a-round." << std::endl;
 		#endif
-		return false;
+		FocusAt=LastFocused;
+		//return false;
 		}
 	#ifdef _DEBUG_SELECT_
 	std::cout << "Selector captured at CursorOffset()=" << std::dec << CursorOffset() << "\t select->StartOffset:" <<  select->StartOffset << std::endl;
@@ -352,27 +369,41 @@ void inline HexEditorCtrl::ClearPaint( void ){
 
 void HexEditorCtrl::PreparePaintTAGs( void ){//TagElement& TAG ){
 	TagHideAll();
-	hex_ctrl->TagArray.Clear();
-	text_ctrl->TagArray.Clear();
+	WX_CLEAR_ARRAY(hex_ctrl->TagArray);
+	WX_CLEAR_ARRAY(text_ctrl->TagArray);
 
-	MainTagArray.Sort( TagElement::TagCompare );
-	TagElement *TAG;
-	for( unsigned i = 0 ; i < MainTagArray.Count() ; i ++ ){	//Painting all TAGs here.
-		TAG = MainTagArray.Item(i);// For debugging
-		PushTAGToControls(TAG);
-		}
+	//MainTagArray.Sort( TagElement::TagCompare );
+	//for( unsigned i = 0 ; i < MainTagArray.Count() ; i ++ )	//Painting all TAGs here.
+	//	PushTAGToControls(MainTagArray.Item(i));
+	PaintTAGsPrefilter( MainTagArray );
 
-	HighlightArray.Sort( TagElement::TagCompare );
-	for( unsigned i = 0 ; i < HighlightArray.Count() ; i ++ ){	//Just highlighting required sections.
-		TAG = HighlightArray.Item(i);// For debugging
-		PushTAGToControls(TAG);
-		}
+	//HighlightArray.Sort( TagElement::TagCompare );
+   //for( unsigned i = 0 ; i < HighlightArray.Count() ; i ++ )	//Just highlighting required sections.
+	//  PushTAGToControls(HighlightArray.Item(i));
+	PaintTAGsPrefilter( HighlightArray );
 
-	CompareArray.Sort( TagElement::TagCompare );
-	for( unsigned i = 0 ; i < CompareArray.Count() ; i ++ ){	//Just highlighting diff sections.
-		TAG = CompareArray.Item(i);// For debugging
-		PushTAGToControls(TAG);
-		}
+	//CompareArray.Sort( TagElement::TagCompare );
+	//for( unsigned i = 0 ; i < CompareArray.Count() ; i ++ )	//Just highlighting diff sections.
+	//	PushTAGToControls(CompareArray.Item(i));
+	PaintTAGsPrefilter( CompareArray );
+	}
+
+void HexEditorCtrl::PaintTAGsPrefilter( ArrayOfTAG& Arr ){
+	unsigned c = Arr.Count();
+	if( c==0 )
+		return;
+
+	unsigned s = 0;
+	if (c>0)
+	for( ; page_offset > (Arr.Item(s))->end and s<c-1; s++ );
+	unsigned e=s;
+	if (c>0)
+	for( ; page_offset + GetByteCount() > Arr.Item(e)->start and e<c-1 ; e++ );
+//#ifdef _DEBUG_
+	std::cout << "Tags needed between : " << s << " - " << e << std::endl;
+//#endif //_DEBUG_
+	for( unsigned i=s; i<=e ;i++)
+		PushTAGToControls( Arr.Item(i) );
 	}
 
 //This functions move tags to local hex and text ctrls.
@@ -476,7 +507,7 @@ void HexEditorCtrl::ControlShow( panels control, bool show ){
 		}
 	else if(control == HEX_CTRL ){
 		hex_ctrl->Show( show );
-		m_static_adress->Show(show);
+		m_static_address->Show(show);
 		}
 	else if(control == TEXT_CTRL ){
 		text_ctrl->Show( show );
@@ -571,7 +602,7 @@ void HexEditorCtrl::OnResize( wxSizeEvent &event ){
 
 	hex_ctrl->SetMinSize( wxSize( hex_x, y ));
 //	hex_ctrl->SetSize( wxSize( hex_x, y ));
-	m_static_adress->SetMinSize( wxSize(hex_x, m_static_offset->GetSize().GetY()) ) ;
+	m_static_address->SetMinSize( wxSize(hex_x, m_static_offset->GetSize().GetY()) ) ;
 
 	text_ctrl->SetMinSize( wxSize( text_x, y ));
 //	text_ctrl->SetSize( wxSize( text_x, y ));
@@ -579,8 +610,9 @@ void HexEditorCtrl::OnResize( wxSizeEvent &event ){
 
 	//Preparing Sizer
 	wxFlexGridSizer* fgSizer1 = new wxFlexGridSizer( 2, 4, 0, 0 );
+#if 1
 	fgSizer1->Add( m_static_offset, 0, wxALIGN_CENTER|wxLEFT, 5 );
-	fgSizer1->Add( m_static_adress, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxLEFT, 2 );
+	fgSizer1->Add( m_static_address, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxLEFT, 2 );
 	fgSizer1->Add( m_static_byteview, 0, wxALIGN_CENTER|wxALL, 0 );
 	fgSizer1->Add( m_static_null, 0, wxALIGN_CENTER, 3 );
 	fgSizer1->Add( offset_ctrl, 0, wxALIGN_CENTER|wxALL|wxEXPAND, 0 );
@@ -588,7 +620,20 @@ void HexEditorCtrl::OnResize( wxSizeEvent &event ){
 	fgSizer1->Add( text_ctrl, 0, wxALIGN_CENTER|wxALL|wxEXPAND, 0 );
 	fgSizer1->Add( offset_scroll_real, 0, wxEXPAND, 0 );
 
-//	offset_ctrl->BytePerLine = BytePerLine(); //Not needed, Updated via ReadFromBuffer
+#else
+	fgSizer1->Add( m_static_offset, 0, wxALIGN_CENTER|wxLEFT, 0 );
+	fgSizer1->Add( m_static_address, 0, wxALIGN_CENTER|wxALIGN_CENTER_VERTICAL|wxLEFT, 0 );
+	fgSizer1->Add( m_static_byteview, 0, wxALIGN_CENTER|wxALL, 0 );
+	fgSizer1->Add( m_static_null, 0, wxALIGN_CENTER, 0 );
+	fgSizer1->Add( offset_ctrl, 0, wxALIGN_CENTER|wxALL|wxEXPAND, 0 );
+	fgSizer1->Add( hex_ctrl, 0, wxALIGN_CENTER|wxALL, 0 );
+	fgSizer1->Add( text_ctrl, 0, wxALIGN_CENTER|wxALL|wxEXPAND, 0 );
+	fgSizer1->Add( offset_scroll_real, 0, wxEXPAND, 0 );
+
+
+	fgSizer1->AddGrowableCol(1,1);
+//	fgSizer1->AddGrowableRow(1,1);
+#endif
 
 	this->SetSizer( fgSizer1 );
 	this->Layout();
@@ -602,25 +647,21 @@ void HexEditorCtrl::OnResize( wxSizeEvent &event ){
 
 	//Formating Hex and byteview column labels
 	//This needed bellow hex_ctrl->ChangeSize() because it's updates the IsDenied function.
-	wxString adress,byteview;
+	wxString address,byteview,temp_address;
 
-	int h=0;
 	for( int j = 0 ; j < ByteShownPerLine ; j++ ){
 		byteview << wxString::Format( wxT("%01X"), j%0x10 );
-
-		while(hex_ctrl->IsDenied(h)){
-			adress << wxT(" ");
-			h++;
-			}
-
-		adress << wxString::Format( wxT("%02X"), j%0x100 );
-		h+=2;
+		temp_address << wxString::Format( wxT("%02X"), j%0x100 );
 		}
 
-	if(adress.Last()==' ')
-		adress.RemoveLast();	//Remove last ' ' for unwrap
+	//Adjusting custom hex formatting bar - Converting 00010203 -> 00 01 02 03 for "xx " format.
+	for( int x = 0, i=0 ; x < hex_x and i < temp_address.Len() ; x++ )
+		if(hex_ctrl->IsDenied(x))
+			address << wxT(" ");
+		else
+			address << temp_address[i++];
 
-	m_static_adress->SetLabel(adress);
+	m_static_address->SetLabel(address);
 	m_static_byteview->SetLabel( byteview );
 
 #ifdef _DEBUG_SIZE_
@@ -635,7 +676,7 @@ void HexEditorCtrl::OnResize( wxSizeEvent &event ){
 //------EVENTS---------//
 void HexEditorCtrl::OnMouseLeft(wxMouseEvent& event){
 	select->SetState( false );
-	ClearPaint();
+
 	if( event.GetEventObject() == hex_ctrl ){
 		hex_ctrl->SetFocus();
 		focus=HEX_CTRL;
@@ -649,6 +690,8 @@ void HexEditorCtrl::OnMouseLeft(wxMouseEvent& event){
 	else if( event.GetEventObject() == offset_ctrl ){
 		event.Skip(); //to lower level for copy offset to clipboard
 		}
+
+	ClearPaint(); //redraw cursor shadow after movement.
 	}
 
 void HexEditorCtrl::OnMouseMove( wxMouseEvent& event ){
@@ -687,6 +730,7 @@ void HexEditorCtrl::OnMouseSelectionEnd( wxMouseEvent& event ){
 	}
 
 void HexEditorCtrl::OnMouseRight( wxMouseEvent& event ){
+	//Not used code. HexEditor::OnMouseRight( wxMouseEvent& event ) used instead.
 	if(event.GetEventObject() == hex_ctrl)
 		LastRightClickAt = hex_ctrl->PixelCoordToInternalPosition( event.GetPosition() )/2;
 	else if(event.GetEventObject() == text_ctrl)
@@ -707,9 +751,19 @@ void HexEditorCtrl::OnMouseRight( wxMouseEvent& event ){
 	ShowContextMenu( event );
 	}
 
+void HexEditorCtrl::OnFocus( wxFocusEvent& event){
+#ifdef _DEBUG_
+	std::cout << "HexEditorCtrl::OnFocus( wxFocusEvent& event ) \n" ;
+#endif
+	if( event.GetWindow() == hex_ctrl or
+		 event.GetWindow() == text_ctrl  )
+		LastFocused=event.GetWindow();
+	event.Skip();//let wxHexCtrl::Focus set the cursor
+	}
+
 void HexEditorCtrl::OnKillFocus( wxFocusEvent& event){
-#ifdef _DEBUG_MOUSE_
-	std::cout << "HexEditorCtrl::OnKillFocus( wxMouseEvent& event ) \n" ;
+#ifdef _DEBUG_
+	std::cout << "HexEditorCtrl::OnKillFocus( wxFocusEvent& event ) \n" ;
 #endif
 	TagHideAll();
 	event.Skip();
@@ -812,7 +866,7 @@ bool HexEditorCtrl::LoadTAGS( wxFileName flnm ){
 
 				while (child) {
 					if (child->GetName() == wxT("TAG")) {
-						wxString propvalue = child->GetPropVal(wxT("id"), wxT("default-value"));
+						wxString propvalue = child->GetPropVal(wxT("id"), wxEmptyString);
 	#ifdef _DEBUG_TAG_
 						std::cout << "TAG ID:" << propvalue.ToAscii() << " readed.\n";
 	#endif
@@ -849,6 +903,7 @@ bool HexEditorCtrl::LoadTAGS( wxFileName flnm ){
 						}
 					child = child->GetNext();
 					}
+				MainTagArray.Sort(TagElementSort);
 				PreparePaintTAGs();
 				ClearPaint();
 				text_ctrl->RePaint();
@@ -875,6 +930,8 @@ void HexEditorCtrl::MoveTAGS( uint64_t location, int64_t size ){
 
 bool HexEditorCtrl::SaveTAGS( wxFileName flnm ){
 	if( MainTagArray.Count() ==  0){
+//		if( wxFileName::FileName( flnm.GetFullPath() ).FileExists() )
+//			wxRemoveFile( flnm.GetFullPath() );
 		if( wxFileName::FileName( flnm.GetFullPath() << wxT(".tags") ).FileExists() )
 			wxRemoveFile( flnm.GetFullPath() << wxT(".tags") );
 		return false;
@@ -885,9 +942,11 @@ bool HexEditorCtrl::SaveTAGS( wxFileName flnm ){
 		wxXmlNode *node_Root = new wxXmlNode( NULL, wxXML_ELEMENT_NODE, wxT("wxHexEditor_XML_TAG"), wxEmptyString, NULL , NULL);
 
 		wxXmlProperty *prop_filename = new wxXmlProperty( wxT("path"), flnm.GetFullPath(), NULL);
-		wxXmlNode *node_File = new wxXmlNode( node_Root, wxXML_ELEMENT_NODE, wxT("filename"), flnm.GetFullPath(), prop_filename , NULL);
+		wxXmlNode *node_File = new wxXmlNode( node_Root, wxXML_ELEMENT_NODE, wxT("filename"), wxEmptyString, prop_filename , NULL);
 
-		for(unsigned i = 0 ; i < MainTagArray.Count() ; i++ ){
+		MainTagArray.Sort(TagElementSort);
+		for(signed i = MainTagArray.Count()-1 ; i>=0 ; i-- ){
+			//Used reverse order for make XML offsets increasing.
 			TagElement *TAG = MainTagArray.Item(i);
 
 			wxXmlProperty *ID = new wxXmlProperty( wxT("id"), wxString::Format(wxT("%d"),i), NULL );
@@ -896,21 +955,23 @@ bool HexEditorCtrl::SaveTAGS( wxFileName flnm ){
 			wxXmlNode *element_NoteColour		= new wxXmlNode( node_Tag, wxXML_ELEMENT_NODE, wxT("note_colour"), wxEmptyString, NULL, NULL);
 			new wxXmlNode( element_NoteColour, wxXML_TEXT_NODE, wxT("note_colour"), TAG->NoteClrData.GetColour().GetAsString(wxC2S_HTML_SYNTAX), NULL, NULL);
 
-			wxXmlNode *element_FontColour		= new wxXmlNode( node_Tag, wxXML_ELEMENT_NODE, wxT("font_colour"), TAG->FontClrData.GetColour().GetAsString(wxC2S_HTML_SYNTAX), NULL, element_NoteColour);
+			wxXmlNode *element_FontColour		= new wxXmlNode( node_Tag, wxXML_ELEMENT_NODE, wxT("font_colour"), wxEmptyString, NULL, element_NoteColour);
 			new wxXmlNode( element_FontColour, wxXML_TEXT_NODE, wxT("font_colour"), TAG->FontClrData.GetColour().GetAsString(wxC2S_HTML_SYNTAX), NULL, NULL);
 
-			wxXmlNode *element_TagText		= new wxXmlNode( node_Tag, wxXML_ELEMENT_NODE, wxT("tag_text"), TAG->tag, NULL, element_FontColour);
+			wxXmlNode *element_TagText		= new wxXmlNode( node_Tag, wxXML_ELEMENT_NODE, wxT("tag_text"), wxEmptyString, NULL, element_FontColour);
 			new wxXmlNode( element_TagText, wxXML_TEXT_NODE, wxT("tag_text"), TAG->tag, NULL, NULL);
 
-			wxXmlNode *element_End 			= new wxXmlNode( node_Tag, wxXML_ELEMENT_NODE, wxT("end_offset"), wxString::Format(wxT("%ld"),TAG->end ) , NULL, element_TagText);
-			new wxXmlNode( element_End, wxXML_TEXT_NODE, wxT("end_offset"), wxString::Format(wxT("%ld"),TAG->end ) , NULL, NULL);
+			wxXmlNode *element_End 			= new wxXmlNode( node_Tag, wxXML_ELEMENT_NODE, wxT("end_offset"), wxEmptyString, NULL, element_TagText);
+			new wxXmlNode( element_End, wxXML_TEXT_NODE, wxT("end_offset"), wxString::Format(wxT("%" wxLongLongFmtSpec "u"),TAG->end ) , NULL, NULL);
 
-			wxXmlNode *element_Start		= new wxXmlNode( node_Tag, wxXML_ELEMENT_NODE, wxT("start_offset"), wxString::Format(wxT("%ld"), TAG->start ), NULL, element_End);
-			new wxXmlNode( element_Start, wxXML_TEXT_NODE, wxT("start_offset"), wxString::Format(wxT("%ld"), TAG->start ), NULL, NULL);
+			wxXmlNode *element_Start		= new wxXmlNode( node_Tag, wxXML_ELEMENT_NODE, wxT("start_offset"), wxEmptyString, NULL, element_End);
+			new wxXmlNode( element_Start, wxXML_TEXT_NODE, wxT("start_offset"), wxString::Format(wxT("%" wxLongLongFmtSpec "u"), TAG->start ), NULL, NULL);
 			}
 		doc.SetFileEncoding( wxT("UTF-8") );
 		doc.SetRoot( node_Root );
-		return doc.Save(flnm.GetFullPath().Append(wxT(".tags")));
+		if( not flnm.GetFullName().Lower().EndsWith(wxT(".tags")) )
+			return doc.Save(flnm.GetFullPath() + wxT(".tags"));
+		return doc.Save(flnm.GetFullPath());
 		}
 	}
 
@@ -935,6 +996,7 @@ void HexEditorCtrl::SetLocalHexInsertionPoint( int hex_location ){	//Sets positi
 #endif // _DEBUG_CARET_
 	text_ctrl->SetInsertionPoint( hex_location/2 );
 	hex_ctrl->SetInsertionPoint( hex_location );
+
 	}
 uint64_t HexEditorCtrl::CursorOffset( void ){
 	return GetLocalInsertionPoint() + page_offset;
@@ -1103,3 +1165,4 @@ void wxHugeScrollBar::OnOffsetScroll( wxScrollEvent& event ){
 #endif
 	event.Skip();
 	}
+
