@@ -87,9 +87,10 @@ wxArrayString GetDeviceList( bool WithPartitions=true){
 HexEditorFrame::HexEditorFrame( wxWindow* parent,int id ):
 				HexEditorGui( parent, id, wxString(_T("wxHexEditor ")) << _T(_VERSION_STR_ )){
 	#if defined( _DEBUG_ ) && defined( __WXMSW__ )
-	debugFrame = new DebugFrame(NULL, wxID_ANY);
-	debugFrame->Show();
-	debugRedirector = new wxStreamToTextRedirector(debugFrame->m_textCtrl);
+	int no = AttachConsole(ATTACH_PARENT_PROCESS); //doesn't work!, don't know why....
+	///Use LDFLAGS="Wl,--subsystem,console -mconsole" to have debug window
+	//wxMessageBox( wxString::Format(" Attach %d", no)    , "none" );
+	//AllocConsole();
 	#endif
 
 	wxIcon wxHexEditor_ICON ( wxhex_xpm );
@@ -124,9 +125,6 @@ HexEditorFrame::HexEditorFrame( wxWindow* parent,int id ):
 		 w = pConfig->Read(_T("ScreenW"), 600),
 		 h = pConfig->Read(_T("ScreenH"), 400);
 
-	bool fullScreen;
-	pConfig->Read(_T("ScreenFullScreen"), &fullScreen, false);
-
 	// Normalizing for avoid screen disapperaring
 	wxSize dsz = wxGetDisplaySize();
 	x = x < 0 ? 0 : x < dsz.x ? x : dsz.x - w;
@@ -136,8 +134,14 @@ HexEditorFrame::HexEditorFrame( wxWindow* parent,int id ):
 	Move(x, y);
 	SetClientSize(w, h);
 
-	Maximize(fullScreen);
+	//	int startup_number=1;
+//	pConfig->Read(_T("Startup"), &startup_number, 1);
+//	pConfig->Write(_T("Startup"), &startup_number, 1);
+//
 
+	bool fullScreen;
+	pConfig->Read(_T("ScreenFullScreen"), &fullScreen, false);
+	Maximize(fullScreen);
 
 #if defined __WXMAC__ || defined __WXMSW__
 	wxArtProvider::Push(new HexEditorArtProvider); //Using similar MacOSX icons. Much more better than wx ones...
@@ -154,7 +158,7 @@ HexEditorFrame::HexEditorFrame( wxWindow* parent,int id ):
 	PrepareAUI();
 
 	MyAUI->Update();
-	this->Connect( XORVIEW_EVENT, wxEVT_UPDATE_UI, wxUpdateUIEventHandler( HexEditorFrame::OnUpdateUI ) );
+   this->Connect( XORVIEW_EVENT, wxEVT_UPDATE_UI, wxUpdateUIEventHandler( HexEditorFrame::OnUpdateUI ) );
 	this->Connect( SELECT_EVENT, wxEVT_UPDATE_UI, wxUpdateUIEventHandler( HexEditorFrame::OnUpdateUI ) );
 	this->Connect( UNREDO_EVENT, wxEVT_UPDATE_UI, wxUpdateUIEventHandler( HexEditorFrame::OnUpdateUI ) );
 	this->Connect( TAG_CHANGE_EVENT, wxEVT_UPDATE_UI, wxUpdateUIEventHandler( HexEditorFrame::OnUpdateUI ) );
@@ -198,13 +202,6 @@ HexEditorFrame::HexEditorFrame( wxWindow* parent,int id ):
 	}
 
 HexEditorFrame::~HexEditorFrame(){
-	#if defined( _DEBUG_ ) && defined( __WXMSW__ )
-	if( debugFrame not_eq NULL )
-		debugFrame->Destroy();
-	if(debugRedirector not_eq NULL)
-		delete debugRedirector;
-	#endif
-
 	this->Disconnect( XORVIEW_EVENT, wxEVT_UPDATE_UI, wxUpdateUIEventHandler( HexEditorFrame::OnUpdateUI ) );
 	this->Disconnect( SELECT_EVENT, wxEVT_UPDATE_UI, wxUpdateUIEventHandler( HexEditorFrame::OnUpdateUI ) );
 	this->Disconnect( UNREDO_EVENT, wxEVT_UPDATE_UI, wxUpdateUIEventHandler( HexEditorFrame::OnUpdateUI ) );
@@ -236,9 +233,13 @@ HexEditorFrame::~HexEditorFrame(){
 	pConfig->Write(_T("ScreenW"), (long) w);
 	pConfig->Write(_T("ScreenH"), (long) h);
 	pConfig->Write(_T("ScreenFullScreen"), IsMaximized());
+
+	pConfig->Write(wxT("LastPerspective"), MyAUI->SavePerspective() );
+
 	pConfig->Flush();
 
 	MyNotebook->Destroy();
+	MyAUI->UnInit();
 	}
 
 void HexEditorFrame::PrepareAUI( void ){
@@ -255,6 +256,8 @@ void HexEditorFrame::PrepareAUI( void ){
 
 	MyAUI -> AddPane( MyNotebook, wxAuiPaneInfo().
 			CaptionVisible(false).
+			Name(wxT("Notebook")).
+			Caption(wxT("Notebook")).
 			MinSize(wxSize(150,100)).
 			CloseButton(false).
 			Center().Layer(1) );
@@ -295,36 +298,48 @@ void HexEditorFrame::PrepareAUI( void ){
 	Toolbar->AddTool(wxID_CUT, _("Cut Block"), wxArtProvider::GetBitmap(wxART_CUT, wxART_TOOLBAR), _("Cuts selected block and copies to clipboard"));
 	Toolbar->AddTool(wxID_DELETE, _("Delete Block"), wxArtProvider::GetBitmap(wxART_DELETE, wxART_TOOLBAR), _("Deletes selected block"));
 	Toolbar->AddTool(idInsert, _("Insert Block"), wxArtProvider::GetBitmap(wxART_GO_DOWN, wxART_TOOLBAR), _("Insert"));
-
-//  Toolbar->SetCustomOverflowItems(prepend_items, append_items);
+/*
+#ifdef _WX_AUIBAR_H_
+	Toolbar->AddStretchSpacer();
+#else
+	Toolbar->AddStretchableSpacer();
+#endif
+*/
    Toolbar->Realize();
-	mbar->Check( idToolbar, true );
+
+	//MyAUI->LoadPerspective()???
+	MyAUI -> AddPane(Toolbar, wxAuiPaneInfo().
+                  Name(wxT("Toolbar")).
+						Caption(_("Toolbar")).
+                  ToolbarPane().Top().
+                  LeftDockable(false).RightDockable(false));
 
 	MyTagPanel = new TagPanel( this, -1 );
 	MyAUI -> AddPane( MyTagPanel, wxAuiPaneInfo().
-					Caption(wxT("TagPanel")).
+					Name(wxT("TagPanel")).
+					Caption(_("TagPanel")).
 					TopDockable(false).
 					BottomDockable(false).
 					MinSize(wxSize(70,100)).
 					BestSize(wxSize(140,100)).
 					Show(false).
 					Right().Layer(1) );
-	mbar->Check( idTagPanel, true );
 
 	MyDisassemblerPanel = new DisassemblerPanel( this, -1 );
 	MyAUI -> AddPane( MyDisassemblerPanel, wxAuiPaneInfo().
-					Caption(wxT("Disassembler Panel")).
+					Name(wxT("Disassembler Panel")).
+					Caption(_("Disassembler Panel")).
 					TopDockable(false).
 					BottomDockable(false).
 					MinSize(wxSize(70,100)).
 					BestSize(wxSize(140,100)).
 					Show(false).
 					Right().Layer(1) );
-	mbar->Check( idTagPanel, true );
 
 	MySearchPanel = new SearchPanel( this, -1 );
    //Created under OnUpdateUI
    MyAUI -> AddPane( MySearchPanel, wxAuiPaneInfo().
+				Name(wxT("Search Results")).
 				Caption(_("Search Results")).
 				TopDockable(false).
 				BottomDockable(false).
@@ -336,6 +351,7 @@ void HexEditorFrame::PrepareAUI( void ){
 	MyComparePanel = new ComparePanel( this, -1 );
    //Created under OnUpdateUI
    MyAUI -> AddPane( MyComparePanel, wxAuiPaneInfo().
+				Name(wxT("Comparison Results")).
 				Caption(_("Comparison Results")).
 				TopDockable(false).
 				BottomDockable(false).
@@ -344,32 +360,31 @@ void HexEditorFrame::PrepareAUI( void ){
 				Show(false).
 				Right().Layer(1) );
 
-   MyAUI -> AddPane(Toolbar, wxAuiPaneInfo().
-                  Name(wxT("ToolBar")).Caption(_("Toolbar")).
-                  ToolbarPane().Top().
-                  LeftDockable(false).RightDockable(false));
-
 	MyInterpreter = new DataInterpreter( this, -1 );
 	MyAUI -> AddPane( MyInterpreter, wxAuiPaneInfo().
+					Name(wxT("DataInterpreter")).
 					Caption(_("DataInterpreter")).
 					TopDockable(false).
 					BottomDockable(false).
 					BestSize(wxSize(174,218)).
 					Resizable(false).
+					Show(true).
 					Left().Layer(1).Position(0) );
-	mbar->Check( idInterpreter, true );
 
 	MyInfoPanel = new InfoPanel( this, -1 );
 	MyAUI -> AddPane( MyInfoPanel, wxAuiPaneInfo().
+					Name(wxT("InfoPanel")).
 					Caption(_("InfoPanel")).
 					TopDockable(false).
 					BottomDockable(false).
 					BestSize(wxSize(140,140)).
+					Show(true).
 					//Resizable(false).
 					Left().Layer(1).Position(1) );
-	mbar->Check( idInfoPanel, true );
-
 
+   wxString tempStr;
+   wxConfigBase::Get()->Read(_T("LastPerspective"), &tempStr, wxEmptyString);
+	MyAUI->LoadPerspective( tempStr );
 
 	ActionDisabler();
 	MyNotebook->SetDropTarget( new DnDFile( this ) );
@@ -411,12 +426,14 @@ void HexEditorFrame::ActionDisabler( void ){
 	Toolbar->Refresh();
 	}
 
-HexEditor* HexEditorFrame::OpenFile(wxFileName filename){
+HexEditor* HexEditorFrame::OpenFile(wxFileName filename, bool openAtRight){
 	HexEditor *x = new HexEditor(MyNotebook, -1, statusBar,	MyInterpreter,	MyInfoPanel, MyTagPanel, MyDisassemblerPanel );
 	x->Hide();//Hiding hex editor for avoiding visual artifacts on loading file...
 	if(x->FileOpen( filename )){
-		MyNotebook->AddPage( x, x->GetFileName().GetFullPath(), true );
+		MyNotebook->AddPage( x, x->GetFileName().GetFullName(), true );
 		x->Show();
+		if(openAtRight)
+			MyNotebook->Split( MyNotebook->GetSelection() , wxRIGHT);
 
 		bool autoShowTagsSwitch;
 		wxConfigBase::Get()->Read( _T("AutoShowTagPanel"), &autoShowTagsSwitch, true );
@@ -508,7 +525,6 @@ void HexEditorFrame::OnMenuEvent( wxCommandEvent& event ){
 			OpenFile( flname );
 			filediag->Destroy();
 			}
-		return; // Without this, wxID_NEW retriggers this function again under wxMSW
 		}
 	else if( event.GetId() == wxID_OPEN ){
 		wxFileDialog filediag(this,
@@ -523,14 +539,13 @@ void HexEditorFrame::OnMenuEvent( wxCommandEvent& event ){
 			OpenFile( flname );
 			filediag.Destroy();
 			}
-		return; // Without this, wxID_OPEN retriggers this function again under wxMSW
 		}
 	else if( event.GetId() >= MyFileHistory->GetBaseId() and event.GetId() <= MyFileHistory->GetBaseId()+MyFileHistory->GetCount()-1){
 		wxString filename = MyFileHistory->GetHistoryFile( event.GetId() - MyFileHistory->GetBaseId() );
 		OpenFile( filename );
 		}
-	else{
-		if( MyNotebook->GetPageCount() ){
+	//If there some file(s) are open
+	else if( MyNotebook->GetPageCount() ){
 			HexEditor *MyHexEditor = GetActiveHexEditor();
 			if( MyHexEditor != NULL ){
 				switch( event.GetId() ){
@@ -564,9 +579,12 @@ void HexEditorFrame::OnMenuEvent( wxCommandEvent& event ){
 																					wxT("*.tags"),
 																					wxFD_OPEN|wxFD_CHANGE_DIR,
 																					wxDefaultPosition);
-												if(wxID_OK == filediag.ShowModal())
+												if(wxID_OK == filediag.ShowModal()){
 													if( !MyHexEditor->LoadTAGS( wxFileNameFromPath(filediag.GetPath()) ) )
 														wxMessageBox( wxString(_( "TAGS cannot imported from ")).Append( filediag.GetPath() ),_("Error"), wxOK|wxICON_ERROR, this );
+													else
+														MyTagPanel->Set( MyHexEditor->MainTagArray );
+													}
 												break;
 												}
 					case idExportTAGs:	{
@@ -578,9 +596,7 @@ void HexEditorFrame::OnMenuEvent( wxCommandEvent& event ){
 																					wxFD_SAVE|wxFD_OVERWRITE_PROMPT|wxFD_CHANGE_DIR,
 																					wxDefaultPosition);
 													if(wxID_OK == filediag.ShowModal()){
-														if( wxFileNameFromPath(filediag.GetPath()).Lower().EndsWith(wxT(".tags.tags")) )
-															filediag.SetPath( filediag.GetPath().RemoveLast( 4 ));
-														if( !MyHexEditor->SaveTAGS( wxFileNameFromPath(filediag.GetPath()) ) )
+														if( !MyHexEditor->SaveTAGS( filediag.GetPath() ) )
 															wxMessageBox( wxString(_( "TAGs cannot exported to ")).Append( filediag.GetPath() ),_("Error"), wxOK|wxICON_ERROR, this );
 														}
 													}
@@ -611,9 +627,7 @@ void HexEditorFrame::OnMenuEvent( wxCommandEvent& event ){
 					}
 				}
 			}
-		return; // Without this, the event retriggers this function again under wxMSW
-		}
-	event.Skip();
+	event.Skip(false);
 	}
 
 void HexEditorFrame::OnToolsMenu( wxCommandEvent& event ){
@@ -674,26 +688,18 @@ void HexEditorFrame::OnDevicesMenu( wxCommandEvent& event ){
 
 void HexEditorFrame::OnOptionsMenu( wxCommandEvent& event ){
 	if( event.GetId() == wxID_PREFERENCES){
+		wxString OldLang = wxConfig::Get()->Read(_T("Language"), wxEmptyString);
 		PreferencesDialog *prefdlg = new PreferencesDialog( this );
 		prefdlg->ShowModal();
 		prefdlg->Destroy();
 
-		if ( ! wxConfig::Get()->Read(_T("Language")).IsEmpty() ) {
-			int lang = wxConfigBase::Get()->Read(_T("Language"), -1) ;
-			int z=wxGetLocale()->GetLanguage();
-			if ( lang != -1 )
-				if ( wxGetLocale()->GetLanguage() != lang) {
-					if( lang == 0 && wxGetLocale()->GetSystemLanguage() == wxGetLocale()->GetLanguage() )	//prevents default redraw
-						return;
-					else {
-						wxMessageBox( _("Please restart program for changes."), _("Info") );
+		if ( wxConfig::Get()->Read(_T("Language")) != OldLang ) {
+			wxMessageBox( _("Please restart program for changes."), _("Info") );
 //						delete single_inst_checker;
 //						single_inst_checker=NULL;
 //						wxGetApp().ReCreateGui();
 //						Destroy();
-						}
-					}
-			}
+				}
 	}
 	HexEditor *MyHexEditor = GetActiveHexEditor();
 	if( MyHexEditor != NULL ) {
@@ -832,13 +838,22 @@ void HexEditorFrame::OnUpdateUI(wxUpdateUIEvent& event){
 	std::cout << "HexEditorFrame::OnUpdateUI(wxUpdateUIEvent& event) ID " << event.GetId() << "\n" ;
 #endif
 // TODO (death#1#): Add idBased approach to decrease overhead!! This slowdowns cursor movement!!!
-	mbar->Check(idInterpreter, MyInterpreter->IsShown());
-	mbar->Check(idInfoPanel, MyInfoPanel->IsShown());
-	mbar->Check(idTagPanel, MyTagPanel->IsShown());
-	mbar->Check(idDisassemblerPanel, MyDisassemblerPanel->IsShown());
-	mbar->Check(idSearchPanel, MySearchPanel->IsShown());
-	mbar->Check(idComparePanel, MyComparePanel->IsShown());
-	mbar->Check(idToolbar, Toolbar->IsShown());
+
+	mbar->Check(idInterpreter,	(MyAUI->GetPane(MyInterpreter).IsFloating()	? MyAUI->GetPane(MyInterpreter).frame->IsShown() : MyInterpreter->IsShown()) );
+	mbar->Check(idInfoPanel,	(MyAUI->GetPane(MyInfoPanel).IsFloating() 	? MyAUI->GetPane(MyInfoPanel).frame->IsShown() : MyInfoPanel->IsShown()) );
+	mbar->Check(idTagPanel,		(MyAUI->GetPane(MyTagPanel).IsFloating()		? MyAUI->GetPane(MyTagPanel).frame->IsShown() : MyTagPanel->IsShown()) );
+	mbar->Check(idDisassemblerPanel,(MyAUI->GetPane(MyDisassemblerPanel).IsFloating() ? MyAUI->GetPane(MyDisassemblerPanel).frame->IsShown() : MyDisassemblerPanel->IsShown()) );
+	mbar->Check(idSearchPanel,	(MyAUI->GetPane(MySearchPanel).IsFloating()	? MyAUI->GetPane(MySearchPanel).frame->IsShown() : MySearchPanel->IsShown()) );
+	mbar->Check(idComparePanel,(MyAUI->GetPane(MyComparePanel).IsFloating()	? MyAUI->GetPane(MyComparePanel).frame->IsShown() : MyComparePanel->IsShown()) );
+	mbar->Check(idToolbar,		(MyAUI->GetPane(Toolbar).IsFloating()			? MyAUI->GetPane(Toolbar).frame->IsShown() : Toolbar->IsShown()) );
+
+//	mbar->Check(idInfoPanel, MyInfoPanel->IsShown());
+//	mbar->Check(idTagPanel, MyTagPanel->IsShown());
+//	mbar->Check(idDisassemblerPanel, MyDisassemblerPanel->IsShown());
+//	mbar->Check(idSearchPanel, MySearchPanel->IsShown());
+//	mbar->Check(idComparePanel, MyComparePanel->IsShown());
+//	mbar->Check(idToolbar, Toolbar->IsShown());
+
 	mbar->Check(idXORView, (MyNotebook->GetPageCount() and GetActiveHexEditor()->IsFileUsingXORKey()));
 	mbar->Enable(idXORView, MyNotebook->GetPageCount() );
 	mbar->Enable(idShowOffset, MyNotebook->GetPageCount() );
@@ -1019,7 +1034,7 @@ void HexEditorFrame::OnNotebookTabSelection( wxAuiNotebookEvent& event ){
 				MyInfoPanel->Set( MyHexEditor->GetFileName(), MyHexEditor->FileLength(), MyHexEditor->GetFileAccessModeString(), MyHexEditor->GetFD(), MyHexEditor->FileGetXORKey() );
 				MyTagPanel->Set( MyHexEditor->MainTagArray );
 				MySearchPanel->Set( MyHexEditor->HighlightArray );
-				MyComparePanel->Set( MyHexEditor->CompareArray );
+				//MyComparePanel->Set( MyHexEditor->CompareArray );
 
 				//Updates visible panels checks
 				mbar->Check( idShowOffset, GetActiveHexEditor()->ControlIsShown( HexEditorCtrl::OFFSET_CTRL ) );
@@ -1049,6 +1064,7 @@ void HexEditorFrame::OnNotebookTabClose( wxAuiNotebookEvent& event ){
 	if( MyNotebook->GetPageCount() ){
 		HexEditor *MyHexEditor = static_cast<HexEditor*>( MyNotebook->GetPage( event.GetSelection() ) );
 		if( MyHexEditor != NULL ){
+		   MyHexEditor->Disconnect( wxEVT_KEY_DOWN,	wxKeyEventHandler(HexEditorFrame::OnKeyDown)  ,NULL, this);
 			if( MyHexEditor->FileClose() ){
 				MyNotebook->DeletePage( event.GetSelection() );
 				// delete MyHexEditor; not neccessery, DeletePage also delete this
